/* -*- mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2002 CodeFactory AB
 * Copyright (C) 2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2002 Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "mrp-interval.h"

struct _MrpInterval {
	mrptime         start;
	mrptime         end;
	
	MrpIntervalType type;

	/* Private */
	guint           ref_count;
};


static void     interval_free (MrpInterval *interval);

static void
interval_free (MrpInterval *interval)
{
        g_return_if_fail (interval != NULL);
        
        g_free (interval);
}

MrpInterval *
mrp_interval_new (mrptime         start,
                  mrptime         end,
                  MrpIntervalType type)
{
        MrpInterval *ret_val;
        
        ret_val = g_new0 (MrpInterval, 1);
        
        ret_val->start     = start;
        ret_val->end       = end;
        ret_val->type      = type;
        ret_val->ref_count = 1;

        return ret_val;
}

MrpInterval *
mrp_interval_copy (MrpInterval *interval)
{
        MrpInterval *ret_val;

        g_return_val_if_fail (interval != NULL, NULL);
        
        ret_val = g_new0 (MrpInterval, 1);
        
        memcpy (ret_val, interval, sizeof (MrpInterval));
        
        return ret_val;
}

MrpInterval *
mrp_interval_ref (MrpInterval *interval)
{
        g_return_val_if_fail (interval != NULL, NULL);
        
        interval->ref_count++;

        return interval;
}

void
mrp_interval_unref (MrpInterval *interval)
{
        g_return_if_fail (interval != NULL);

        interval->ref_count--;
        
        if (interval->ref_count <= 0) {
                interval_free (interval);
        }
}

void
mrp_interval_get_absolute (MrpInterval *interval,
			   mrptime      offset,
			   mrptime     *start,
			   mrptime     *end)
{
	g_return_if_fail (interval != NULL);
	
	*start = interval->start + offset;
	*end   = interval->end + offset;
}

MrpIntervalType
mrp_interval_get_type (MrpInterval *interval)
{
	g_return_val_if_fail (interval != NULL, MRP_INTERVAL_TYPE_WORK);

	return interval->type;
}
	
		       
